pub struct AssetEvent {
pub handle: AssetHandle,
pub kind: AssetEventKind,
pub path: AssetPathStatic,
}Expand description
Represents an event related to an asset, combining a handle and an event kind.
Fields
handle: AssetHandleThe handle of the asset associated with this event.
kind: AssetEventKindThe kind of event that occurred.
path: AssetPathStaticThe path of the asset associated with this event.